# -*- coding: utf-8 -*-

def get_info(cluster_obj, node_address):

    info = dict(address=node_address)
    node_obj = cluster_obj.get_node(node_address)

    if node_obj is not None and node_obj.get_status() == "on" and node_obj.is_valid() is True:
        info['status'] = 'on'
        info['memory'] = node_obj.get_memory_stats()
        info['disk'] = node_obj.get_disk_stats()
        info['build'] = node_obj.get_build()
        stats = node_obj.get_subset_of_statistics([
            "objects", "client_connections", "cluster_size", "migrate_progress_send",
            "migrate_progress_recv", "cluster_integrity", "err_out_of_space", "queue",
            "write_master", "write_prole", "system_swapping", "err_write_fail_prole_unknown",
            "stat_duplicate_operation", "uptime", "system_free_mem_pct", "batch_errors",
            "storage_defrag_records", "stat_write_errs", "free-pct-memory", "free-pct-disk",
            "node_status", 'cluster_visibility'
            ])
        info.update(stats)
        configurations = node_obj.get_subset_of_configurations([
            "proto-fd-max", "client-fd-max", "nsup-threads", "scan-priority", "storage-benchmarks",
            "replication-fire-and-forget"
            ])
        info.update(configurations)
    else:
        info['node_status'] = 'off'

    return info


def get_all_stats(cluster_obj, node_address):

    info = dict(address=node_address)
    node_obj = cluster_obj.get_node(node_address)

    if node_obj is None or not node_obj.is_valid():
        info['node_status'] = "off"
    else:
        stats = node_obj.get_statistics()
        info.update(stats)
        configurations = node_obj.get_configurations()
        info.update(configurations)

    return info


def get_all_config(cluster_obj, node_address):

    info = dict(address=node_address)
    node_obj = cluster_obj.get_node(node_address)

    if node_obj is None or not node_obj.is_valid():
        info['node_status'] = "off"
    else:
        configurations = node_obj.get_configurations()
        info.update(configurations)

    return info


def change_node_status(desired_status, cluster_obj, node_address, username="", password=None):

    info = dict(address=node_address)
    node_obj = cluster_obj.get_node(node_address)

    if node_obj is None:
        info['node_status'] = "off"
    else:
        node_status = node_obj.get_status()
        if node_status == "off" and desired_status == "off":
            info['status'] = "Failue"
            info['error'] = "Node Already Off"
        elif node_status == "on" and desired_status == "on":
            info['status'] = "Failure"
            info['error'] = "Node Already On"
        else:
            status = node_obj.change_status(desired_status, username, password)
            node_cluster_key = node_obj.get_cluster_key()
            if status['status'] == "Success" and desired_status == "on":
                if cluster_obj.get_cluster_key() != node_cluster_key and node_cluster_key != -1:
                    cluster_obj._alert.on_different_cluster_node_add(node_address)
                    cluster_obj.remove_node(node_address)
                    return dict(status="Success")
                elif node_cluster_key == -1:
                    node_obj.set_unknown()
                    return dict(status="Success")
            elif status['status'] == "Success" and desired_status == "off":
                node_obj.set_unknown()
                return dict(status="Success")
            else:
                info.update(status)

    return info


def get_latency(cluster_obj, node_address):

    info = dict(address=node_address, node_status="on")
    node = cluster_obj.get_node(node_address)
    if node is None or node.get_status() == "off" or not node.is_valid():
        info['node_status'] = "off"
    else:
        info['latency'] = node.get_latency()
    return info


def get_latency_history(cluster_obj, node_address):

    info = dict(address=node_address, node_status="on")
    node = cluster_obj.get_node(node_address)

    if node is None or not node.is_valid():
        info['node_status'] = "off"
    elif node.get_status() == "off":
        info['node_status'] = "off"
        info['latency_history'] = list(node.get_latency_history())
    else:
        info['latency_history'] = list(node.get_latency_history())
    return info
